# -*- coding: utf-8 -*-
import pathlib
import os
import sys
import json
import threading
from datetime import datetime
from quesu_csv_type import *
from quesu_csv_file import QuesuFile
from quesu_csv_reference import QuesuReference
from quesu_csv_error import QuesuErrorLogger
import cherrypy
import requests
import zipfile
import zlib
import codecs	
import shutil    
    
class QuesuCsvValidator(threading.Thread):

    def __init__(self,rep_entree) :
        super(QuesuCsvValidator, self).__init__()
        self.repertoire_entree = rep_entree
        cherrypy.log("init repertoire entree"+str(rep_entree))
        

    def run(self):
        # Initialisation des statistiques de validation
        # Pour chaque type de données, l'outil comptabilise :
        # - le nombre de fichiers traités
        # - le nombre d'observations traitées
        # - le nombre total d'erreurs identifiées
        # - le nombre d'erreurs classées "fatale"
        # - le nombre d'erreurs classées "alerte"
        # - le nombre d'erreurs classées "information"
        validation_stats = {
            QUESU_TYPE_PC_ANA:[0, 0, 0, 0, 0, 0],
            QUESU_TYPE_PC_CE: [0, 0, 0, 0, 0, 0],
            QUESU_TYPE_INCONNU: [0, 0, 0, 0, 0, 0]
        }
        start = datetime.now()
        # Chargement des référentiels du Sandre
        cherrypy.log("{0} : chargement des référentiels du SANDRE".format(datetime.now().strftime("%d/%m/%y %H:%M:%S")))
        quesu_references = QuesuReference(self.repertoire_entree).getReferences()
        if quesu_references is None:
            cherrypy.log("{0} : erreur de chargement des référentiels du SANDRE".format(datetime.now().strftime("%d/%m/%y %H:%M:%S")))
            return None
        
        quesu_logger = QuesuErrorLogger(quesu_folder=self.repertoire_entree)
        cheminFichierSynthese = os.path.join(self.repertoire_entree,"synthese.csv")
        try :
            synthese = open(cheminFichierSynthese, "w", encoding="utf-8-sig")
            synthese.write("Fichier de synthese\n")
            synthese.close()
        except PermissionError : 
            cherrypy.log("ERREUR : Probleme de permission pour creer le fichier : "+cheminFichierSynthese)
            return
        totalErreurBloquante = 0
        for csvFile in list(self.repertoire_entree.glob('./*.csv')):
            if str(csvFile).endswith("synthese.csv") :
                cherrypy.log("fichier synthese detect")
                continue
            startFileProcessing = datetime.now()
            synthese = open(cheminFichierSynthese, "a", encoding="utf-8")
            synthese.write("{0}\tcontrôle du fichier {1}\n".format(startFileProcessing.strftime("%d/%m/%y %H:%M:%S"), os.path.basename(str(csvFile))))
            synthese.close()

            file_validator = QuesuFile(csvFile, quesu_logger) # probleme ici
            results = file_validator.validate(quesu_references=quesu_references)

            validation_stats[file_validator.quesu_type][0] += 1
            validation_stats[file_validator.quesu_type][1] += results['line_count']
            validation_stats[file_validator.quesu_type][2] += results['error_count']
            validation_stats[file_validator.quesu_type][3] += results['fatal_count']
            validation_stats[file_validator.quesu_type][4] += results['warning_count']
            validation_stats[file_validator.quesu_type][5] += results['info_count']
            totalErreurBloquante += results['fatal_count']

            stopFileProcessing = datetime.now()
            synthese = open(cheminFichierSynthese, "a", encoding="utf-8")
            
            
            synthese.write("{0}\tdurée du contrôle ({1})\n".format(stopFileProcessing.strftime("%d/%m/%y %H:%M:%S"),
                                                        stopFileProcessing - startFileProcessing))
            synthese.write("{0}\t{1} observations contrôlées\n".format(datetime.now().strftime("%d/%m/%y %H:%M:%S"), results['line_count']))
            synthese.write("{0}\t{1} erreurs détectées : {2} fatales / {3} alertes / {4} informations\n".format(
            datetime.now().strftime("%d/%m/%y %H:%M:%S"), results['error_count'], results['fatal_count'],
            results['warning_count'], results['info_count']))
            synthese.close()
            
        stop = datetime.now()
        synthese = open(cheminFichierSynthese, "a", encoding="utf-8")
        synthese.write("\n{0}\tdurée totale du contrôle ({1})\n".format(stop.strftime("%d/%m/%y %H:%M:%S"), stop - start))
        synthese.write("{0}\t{1} fichiers d'analyses PC contrôlés\n".format(datetime.now().strftime("%d/%m/%y %H:%M:%S"), validation_stats[QUESU_TYPE_PC_ANA][0]))
        synthese.write("{0}\t{1} analyses PC contrôlées\n".format(datetime.now().strftime("%d/%m/%y %H:%M:%S"),
                                                                     validation_stats[QUESU_TYPE_PC_ANA][1]))
        synthese.write("{0}\t{1} erreurs détectées : {2} bloquantes / {3} non bloquantes / {4} informations\n".format(
            datetime.now().strftime("%d/%m/%y %H:%M:%S"), validation_stats[QUESU_TYPE_PC_ANA][2], validation_stats[QUESU_TYPE_PC_ANA][3],
            validation_stats[QUESU_TYPE_PC_ANA][4], validation_stats[QUESU_TYPE_PC_ANA][5]))
        synthese.write("{0}\t{1} fichiers de CEP contrôlés\n".format(datetime.now().strftime("%d/%m/%y %H:%M:%S"),
                                                                     validation_stats[QUESU_TYPE_PC_CE][0]))
        synthese.write("{0}\t{1} CEP contrôlées\n".format(datetime.now().strftime("%d/%m/%y %H:%M:%S"),
                                                           validation_stats[QUESU_TYPE_PC_CE][1]))
        synthese.write("{0}\t{1} erreurs détectées : {2} bloquantes / {3} non bloquantes / {4} informations\n".format(
            datetime.now().strftime("%d/%m/%y %H:%M:%S"), validation_stats[QUESU_TYPE_PC_CE][2],
            validation_stats[QUESU_TYPE_PC_CE][3],
            validation_stats[QUESU_TYPE_PC_CE][4], validation_stats[QUESU_TYPE_PC_CE][5]))
        synthese.write("{0}\t{1} fichiers de type INCONNU contrôlés\n".format(datetime.now().strftime("%d/%m/%y %H:%M:%S"),
                                                              validation_stats[QUESU_TYPE_INCONNU][0]))
        synthese.write("{0}\t{1} CEP contrôlées\n".format(datetime.now().strftime("%d/%m/%y %H:%M:%S"),
                                                   validation_stats[QUESU_TYPE_INCONNU][1]))
        synthese.write("{0}\t{1} erreurs détectées : {2} bloquantes / {3} non bloquantes / {4} informations\n".format(
            datetime.now().strftime("%d/%m/%y %H:%M:%S"), validation_stats[QUESU_TYPE_INCONNU][2],
            validation_stats[QUESU_TYPE_INCONNU][3],
            validation_stats[QUESU_TYPE_INCONNU][4], validation_stats[QUESU_TYPE_INCONNU][5]))

        synthese.write("Voir le détail des erreurs dans les fichiers *_err*.csv du sous répertoire quesu_error\n")
        synthese.close()
        if (totalErreurBloquante>0) :
            statut = "KO"
        else : 
            statut = "OK"
        cherrypy.log("statut de fin : "+statut)    
        # on fait un zip :
        #horodatage = datetime.now().strftime('%d%m%Y%H%M%S')
        chemin_zip = str(self.repertoire_entree)+'/resultat_controle.zip'
        cherrypy.log("creation du zip dans : "+str(chemin_zip))
        zf = zipfile.ZipFile(chemin_zip, mode='w',compression=zipfile.ZIP_DEFLATED)
        try :
            zf.write(cheminFichierSynthese,os.path.basename(cheminFichierSynthese))
            for fichier in list(self.repertoire_entree.glob('quesu_validation/*.*')):
                cherrypy.log("ajout du fichier de validation: "+str(fichier))
                zf.write(fichier,os.path.join("quesu_validation",os.path.basename(fichier)))
            for fichier in list(self.repertoire_entree.glob('quesu_error/*.*')):
                cherrypy.log("ajout du fichier d'anomalies : "+str(fichier))
                zf.write(fichier,os.path.join("quesu_error",os.path.basename(fichier)))
            for fichier in list(self.repertoire_entree.glob('quesu_reference/*.xml')):
                cherrypy.log("ajout du fichier du referentiel : "+str(fichier))
                zf.write(fichier,os.path.join("quesu_reference",os.path.basename(fichier)))                
        finally : 
            zf.close()
        # on supprime les fichiers csv source et les fichiers quesu_reference
        for csvFile in list(self.repertoire_entree.glob('./*.csv')):
            cherrypy.log("fichier : "+str(csvFile))
            cherrypy.log("base : "+str(os.path.basename(csvFile)))
            if str(os.path.basename(csvFile)).upper().startswith("QUESU_CSV_") :
                cherrypy.log("suppression de "+str(csvFile))
                os.remove(os.path.join(self.repertoire_entree,csvFile))
        shutil.rmtree(os.path.join(self.repertoire_entree,"quesu_reference"))        
        # on fait appel au webservice de callback  
        url_callback = cherrypy.config['webservice_retour']+str(self.parametre_url)+"?status="+statut+"&debut="+str(int(start.timestamp())*1000)+"&fin="+str(int(stop.timestamp())*1000)
        try :
            # on n'utilise pas le proxy pour la requête de callback
            session = requests.Session()
            session.trust_env = False
            r = session.get(url_callback)
            cherrypy.log("Appel webService : "+url_callback)
        except :
            cherrypy.log("Connexion impossible au webservice de callback : "+url_callback)
            
    
@cherrypy.expose      
class Webservice(object):

    @cherrypy.expose
    def GET(self,repertoire=None):
        cherrypy.response.headers['Content-Type'] = 'application/json'
        if repertoire is not None :
            # verifier ici que le repertoire existe bien
            csvFolder = pathlib.Path(config_repertoire_alimentation.joinpath(repertoire))
            if not csvFolder.exists() or not csvFolder.is_dir():
                cherrypy.log("ERREUR : repertoire "+repertoire+" introuvable")
                return json.dumps({"status":"01_LANCEMENT_REPERTOIRE_ABSENT"}).encode('utf-8')
            else :
                # cas passant : faire un nouveau thread ?
                monValidator = QuesuCsvValidator(config_repertoire_alimentation.joinpath(repertoire))
                monValidator.parametre_url = repertoire
                monValidator.start()
                return json.dumps({"status":"LANCEMENT_OK"}).encode('utf-8')
        else : 
            # cas où aucun répertoire à vérifier n'est indiqué
            cherrypy.log("ERREUR : aucun repertoire")
            return json.dumps({"status":"02_LANCEMENT_REPERTOIRE_ABSENT"}).encode('utf-8')
    
    def POST(self, repertoire):
        cherrypy.response.headers['Content-Type'] = 'application/json'
        if repertoire is not None :
            # verifier ici que le repertoire existe bien
            csvFolder = pathlib.Path(config_repertoire_alimentation.joinpath(repertoire))
            if not csvFolder.exists() or not csvFolder.is_dir():
                cherrypy.log("ERREUR : repertoire "+repertoire+" introuvable")
                return json.dumps({"status":"03_LANCEMENT_REPERTOIRE_ABSENT"}).encode('utf-8')
            else :
                # cas passant : faire un nouveau thread ?
                monValidator = QuesuCsvValidator(config_repertoire_alimentation.joinpath(repertoire))
                monValidator.parametre_url = repertoire
                monValidator.start()
                return json.dumps({"status":"LANCEMENT_OK"}).encode('utf-8')
        else : 
            # cas où aucun répertoire à vérifier n'est indiqué
            cherrypy.log("ERREUR : aucun repertoire")
            return json.dumps({"status":"04_LANCEMENT_REPERTOIRE_ABSENT"}).encode('utf-8')

    def PUT(self, another_string):
        return "non implemented"

    def DELETE(self):
        return "non implemented"


if __name__ == '__main__':
    cherrypy.log("debut du programme")
    cherrypy.config.update("/applications/www/naiades_ws_python/config/naiades.config")
    referentiels_sandre = pathlib.Path(cherrypy.config['referentiels_sandre'])
    config_repertoire_alimentation = pathlib.Path(cherrypy.config['repertoire_alimentation'])
    if not referentiels_sandre.exists() :
        cherrypy.log("Repertoire sandre introuvable "+cherrypy.config['referentiels_sandre']+" introuvable")
        exit()
    elif not config_repertoire_alimentation.exists() :    
        cherrypy.log("Repertoire entree introuvable "+cherrypy.config['repertoire_alimentation']+" introuvable")
        exit()
    else :    
        #quesu_references = QuesuReference(referentiels_sandre).getReferences()
        #cherrypy.log("fin de chargement du referentiel sandre")
        #cherrypy.tree.mount(WebserviceReferentiel(),"/quesu_referentiel","naiades.config")
        cherrypy.quickstart(Webservice(),"/quesu_csv_validator","/applications/www/naiades_ws_python/config/naiades.config")
		